#ifndef ACTOR_H

#define ACTOR_H

#include "animbitm.h"
#include "animap.h"
#include "script.h"

//@doc

class event;

//@type actorID_t | ID number associated with the actor
typedef word actorID_t;
//@type direction_t | Direction the actor is facing (used most with <c facingActor>)
typedef byte direction_t;

/*
@class The actor class represents actors on an <c animap>.  These can be objects,
players, monsters, devices, explosions, whatever.  The base actor class shows one "face"
and can be animated.  For actors that change their "face" depending on direction of
motion, see <c facingActor>

@base public | yakObject
@base public | animslave
@base public | script
*/
class actor :
  public yakObject,
  public animslave,
  public script {

protected:
  //@access Protected data members
  //@cmember used to check for temporary actors, ie events, explosions, etc.
  bFlag bfIsTempActor;
  //@cmember Flag showing the status of the actor (on or off map)
  bFlag bfIsOnMap;
  //@cmember Tests whether this <c actor> is in front of another <c actor>
  virtual bFlag bfIsInFrontOf( actor * pactTarget ) const;
  //@cmember X coordinate of the actor
  mapCoordX_t mcx;
  //@cmember Y coordinate of the actor
  mapCoordY_t mcy;
  //@cmember If spawned, the address of the spawning actor
  actor * pactSpawning;
  //@cmember If spawning, the address of the spawned actor.
  actor * pactSpawned;
  //@cmember The address of the map the actor is located on
  animap * pmapOwning;
  //@cmember Number of advances since the animBitmap was last incremented.
  int iAdvanceCounter;
  //@cmember Number of advances to wait between frames of animation.
  int iAdvanceDelay;
  //@member SetRefresh | Sets the refresh of all the <c animapSquare>s the <c actor> affects
#ifndef __WATCOMC__
  void actor::SetRefresh( void ) const;
#endif
public:
  //@access Public Data Members
  //@cmember Identity of the actor
  actorID_t aidIdentity;
  //@cmember Direction of the actor, mostly used by <c facingActor>s.
  direction_t drcDirection;
  //@cmember Color to display actor in, if not zero (used to show collisions)
  color_t colCollisionColor;

  //@cmember Default constructor
  actor();
  //@cmember Copy constructor
  actor( const actor * pactCopyFrom );
  //@cmember Default destructor
  virtual ~actor();

  //@member bfIsOnMap | Returns bfIsOnMap data member
  inline bFlag bfGetIsOnMap() const { return bfIsOnMap; };
  //@member iGetAdvanceDelay | Returns iAdvanceDelay data member
  inline int iGetAdvanceDelay() const { return iAdvanceDelay; };
  //@member mcxLeft | Returns left edge of actor
  inline mapCoordX_t mcxLeft() const { return mcx; };
  //@member mcyTop | Returns top edge of actor
  inline mapCoordY_t mcyTop() const { return mcy; };
  //@member mcxRight | Returns right edge of actor
  inline mapCoordX_t mcxRight() const { return mcx + ptCurrent()->iGetWidth() - 1; };
  //@member mcyBottom | Returns bottom edge of actor
  inline mapCoordY_t mcyBottom() const { return mcy + ptCurrent()->iGetHeight() - 1; };
  //@member mcxCenter | Returns x coordinate of the center of the actor
  inline mapCoordX_t mcxCenter() const { return ( mcx + mcxRight() ) / 2; };
  //@member mcyCenter | Returns y coordinate of the center of the actor
  inline mapCoordY_t mcyCenter() const { return ( mcy + mcyBottom() ) / 2; };
  //@member tcxLeft | Returns x coordinate of the top left corner of the actor, in tile coordinates.
  inline tileCoordX_t tcxLeft() const { return mcx / pmapOwning->mcdSquareWidth; };
  //@member tcyTop | Returns y coordinate of the top left corner of the actor, in tile coordinates
  inline tileCoordY_t tcyTop() const { return mcy / pmapOwning->mcdSquareWidth; };
  //@member iInTileX | Returns x coordinate of the top left corner of the actor relative to the top left corner of its tile
  inline int iInTileX() const { return iRollover( mcx, pmapOwning->mcdSquareWidth ); };
  //@member iInTileY | Returns y coordinate of the top left corner of the actor relative to the top left corner of its tile.
  inline int iInTileY() const { return iRollover( mcy, pmapOwning->mcdSquareWidth ); };
  //@member bfGetIsTempActor | returns bfIsTempActor value
  inline bFlag bfGetIsTempActor() const { return bfIsTempActor; };
  //@member bfHitActor | Checks for a pixel-based collision with another actor
  inline bFlag actor::bfHitActor(
    actor * pactTarget,     //@parm A pointer to the target actor
    int iDeltaX = 0,            //@parm Horizontal displacement
    int iDeltaY = 0             //@Parm Vertical displacement
  ) const
  {
    //make a huge call to yakBitmap::bfHitXY
    if ( ptCurrent()->bfHitXY( mcx + iDeltaX, mcy + iDeltaY,
       pactTarget->ptCurrent(), pactTarget->mcx, pactTarget->mcy ) == true ) {
      return true;
    }
    else {
      return false;
    }
  }
#ifdef __WATCOMC__
protected:
  inline void actor::SetRefresh(
    void
  ) const
  {
    //make sure the map pointer isn't NULL.
    ASSERT( pmapOwning != NULL );

    //now loop along the dimensions of the actor and  set refreshes.
    for( int iWCount = ( mcxLeft() / pmapOwning->mcdSquareWidth ); iWCount <= ( mcxRight() / pmapOwning->mcdSquareWidth ); ++iWCount ) {
      for ( int iHCount = ( mcyTop() / pmapOwning->mcdSquareWidth ); iHCount <= ( mcyBottom() / pmapOwning->mcdSquareWidth ); ++iHCount ) {
        pmapOwning->SetRefresh( iRollover( iWCount, pmapOwning->tcxWidth ), iRollover( iHCount, pmapOwning->tcyHeight ), 3 );
      }
    }
  }
public:
#endif
  //@access public
  //@cmember Checks for a pixel-based collision with any actor of a given identity
  virtual bFlag bfHitID( actorID_t aidCheck, actor ** ppactTarget, int iDeltaX = 0, int iDeltaY = 0 ) const;
  //@cmember Checks for a rectangular overlap with any actor of a given identity (not pixel-based)
  virtual bFlag bfOverlapID( actorID_t aidCheck, actor ** ppactTarget, int iDeltaX = 0, int iDeltaY = 0 ) const;

  //@cmember Puts the actor on an <c animap> at a given position
  void PutToMap( animap * pmap, mapCoordX_t mcxNew, mapCoordY_t mcyNew );
  //@cmember Removes the actor from its <c animap>
  void RemoveFromMap( void );
  //@cmember Sets the number of advances the actor will go through before incrementing its animation frame.
  void SetAdvanceDelay( int iNewAdvanceDelay );

  //@cmember "Spawns" another <c actor>
  virtual void Spawn( actor * pactNewSpawned );
  //@cmember Clears the actor's spawning status.
  virtual void ClearSpawn( void );
  //@cmember Turns the actor to a new direction.
  virtual void TurnTo( direction_t drcNewDirection );
  //@cmember Moves the actor a number of <t mapCoord_t> coordinates relative to its current position
  void RelativeMove( int iDeltaSquareX, int iDeltaSquareY );
  //@cmember Moves the actor to a specific <t mapCoord_t> position
  void MoveTo( mapCoordX_t mcx, mapCoordY_t mcy );

  //@cmember Assigns a new animation to the actor.
  void AssignNewAnimBitmap( animBitmap * panm, actorID_t aidNewIdentity );

  //@cmember Advances the actor one line of its <c script>.
  virtual void Advance();
  //@cmember Shows the actor to video memory.  Identical to <mf animslave::Show>
  virtual void Show( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, yakBitmap::showType eShowType = yakBitmap::masked );
  //@cmember Shows the actor to video memory and advances itself one line of script.
  virtual void ShowAndAdvance( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, yakBitmap::showType eShowType = yakBitmap::masked );
};

#endif
