#ifndef ANIMAP_H

#define ANIMAP_H

#include "stddefs.h"
#include "animbitm.h"

//@doc

class actor;
class animapMaintenance;

typedef word actorID_t; //DUPLICATED FROM ACTOR.H
//@type terrainType_t | Type of terrain represented by a given <c animap> square.
typedef word terrainType_t;

/*
@class animapSquares are the unit element of an animap.  They represent a single square
of the owning animap and hold a tile number and a terrain type as well as animated frame
synchronizing and refresh state members.
*/
class animapSquare {
  friend class animap;
  friend class animapMaintenance;
  //@access Private members
  //@cmember Number of the yakBitmap in the owning <c animap>'s tile array.
  word wTileNumber;
  //@cmember Type of terrain in this square
  terrainType_t terTerrain;
  //@cmember Number of frames to refresh this square
  byte bRefresh;
  //@cmember Offset in animation frames of this square if it's "out of sync"
  byte bFrameSync;                                

  //@cmember Default constructor
  animapSquare( void );

  //@cmember Draws the animapSquare to the screen
  void Draw( pageHandle_t pagPage, screenCoordX_t scxRight, screenCoordY_t scyBottom, mapCoord_t mcdSquareWidth, yakBitmap::showType eShowType = yakBitmap::masked );
  //@cmember Draws all the actors in the animapSquare to the screen.
  void DrawActors( pageHandle_t pagPage, screenCoordX_t scxRight, screenCoordY_t scyBottom, yakBitmap::showType eShowType = yakBitmap::masked );
};

#ifdef __386__
typedef int tileCoord_t;
#else
//@type tileCoord_t | Generic <c animap> tile coordinate.
typedef short tileCoord_t;
#endif
//@type tileCoordX_t | X coordinate version of <t tileCoord_t>
typedef tileCoord_t tileCoordX_t;
//@type tileCoordY_t | Y coordinate version of <t tileCoord_t>
typedef tileCoord_t tileCoordY_t;


/*
@class The animap represents a two-dimensional "tiled" map.  It can be used to represent
both top-down tiled worlds (as in the Ultima series) or side-view worlds (as in most platform
side scrollers)
*/
class animap {

friend class actor;
friend class animapSquare;
friend class animapMaintenance;

private:
  //@access Private members
  //@cmember The current map being drawn.
  static animap * pmapCurrent;
  //@cmember The actual array of <c animapSquare>s
  animapSquare ** ppmsqData;
  //@cmember The <c animBitmap>s used for the tiles
  animBitmap * panmTiles;
  //@cmember The names of the <c yakBitmap>s used for the tiles
  filenameString * pacTileNames;
  //@cmember Number of tiles in the animap
  short sNumberOfTiles;                   

  //@cmember Allocates memory for a map of the specified size
  void AllocateMapData( tileCoordX_t tcxWidth, tileCoordY_t tcyHeight );
  //@cmember Frees memory owned by the animap
  void FreeMapData( void );
  //@cmember Copies map data between two maps
  void CopyMapData( byte * pbData, tileCoordX_t tcxWidth, tileCoordY_t tcyHeight );
  //@cmember List of the <c actor>s currently on the map.
  pyList lstActors;               
public:
  //@cmember Width of a map square, in pixels
  short mcdSquareWidth;
  //@cmember Width of the map, in squares
  tileCoordX_t tcxWidth;
  //@cmember Height of the map, in squares
  tileCoordY_t tcyHeight;                 

  //@cmember X coord of last pixel at which plotting started
  screenCoordX_t scxLastDrawLeft;
  //@cmember Y coord of last pixel at which plotting started
  screenCoordY_t scyLastDrawTop;
  //@cmember X coord of last map square at which plotting started
  tileCoordX_t tcxLastDrawLeft;
  //@cmember Y coord of last map square at which plotting started
  tileCoordY_t tcyLastDrawTop;
  //@cmember Width of last draw, in map squares
  int iLastDrawWidth;
  //@cmember Height of last draw, in map squares
  int iLastDrawHeight;

  //@cmember Specific constructor for creating the animap from scratch
  animap( short sNewNumTiles = 1, mapCoord_t mcdNewSquareWidth = 1, tileCoordX_t tcxNewWidth = 1, tileCoordY_t tcyNewHeight = 1 );
  //@cmember Specific constructor which loads the animap from disk
  animap( const char * CszFilename, yakLib * plibForTiles = NULL, yakLib * plibForMap = NULL );
  //@cmember Default destructor
  virtual ~animap( void );
  //@cmember Empties the list of actors on the map.
  void EmptyActorList( void );

  //@cmember gets the name of a numbered tile on the map.
  char * pacGetTileName( int iTileToAccess ) const;
  //@member sGetNumberOfTiles | Returns sNumberOfTiles
  inline short sGetNumberOfTiles( void ) const { return sNumberOfTiles; };
  //@member plstGetActors | Returns a pointer to lstActors
  inline pyList * plstGetActors( void ) { return ( &lstActors ); };

  //@cmember Checks to see if an actor with the given ID is in a given square
  bFlag bfIsInSquare( tileCoordX_t tcx, tileCoordY_t tcy, actorID_t aidCheck, actor ** ppactTarget ) const;
  //@cmember Checks to see if a given tile's terrain type matches given flags
  bFlag bfMatchesTerrainFlags( tileCoordX_t tcx, tileCoordY_t tcy, word wFlags ) const;
  //@cmember Returns the tile number of a given map square
  word wTileNumberTileCoords( tileCoordX_t tcx, tileCoordY_t tcy ) const;
  //@cmember Returns the refresh state of a given map square
  byte bRefresh( tileCoordX_t tcx, tileCoordY_t tcy ) const;
  //@cmember Sets the refresh state of a given map square
  void SetRefresh( tileCoordX_t tcx, tileCoordY_t tcy, byte bNewRefresh );
  //@cmember Returns the terrain type of a given map square
  terrainType_t terTerrainTypeTileCoords( tileCoordX_t tcx, tileCoordY_t tcy ) const;
  //@cmember Returns the terrain type of a given map coordinate
  terrainType_t terTerrainType( mapCoordX_t mcx, mapCoordY_t mcy ) const;
  //@cmember Returns a pointer to the <c animBitmap> data of a given tile number
  animBitmap * panmGetTile( word wTileNumber ) const;
  //@member mcxWidth | Returns width of the map, in <t mapCoordX_t> units
  inline mapCoordX_t mcxWidth() const { return tcxWidth * mcdSquareWidth; };
  //@member mcyHeight | Returns height of the map, in <t mapCoordY_t> units
  inline mapCoordY_t mcyHeight() const { return tcyHeight * mcdSquareWidth; };

  //@cmember Advances the animation of the map tiles and advances every <c actor> on the map.
  void Advance( void );
  //@member Draw | Draws the animap to the screen
  virtual void Draw( pageHandle_t pagPage, screenCoordX_t scxLeft, screenCoordY_t scyTop,
    tileCoordX_t tcxLeft, tileCoordY_t tcyTop, int iDrawWidth, int iDrawHeight,
    yakBitmap::showType eShowTypeForTiles = yakBitmap::masked, yakBitmap::showType eShowTypeForActors = yakBitmap::masked );
  //@member DrawAndAdvance | Draws the animap and then advances it
  void DrawAndAdvance( pageHandle_t pagPage, screenCoordX_t scxLeft, screenCoordY_t scyTop,
    tileCoordX_t tcxLeft, tileCoordY_t tcyTop, int iDrawWidth, int iDrawHeight,
    yakBitmap::showType eShowTypeForTiles = yakBitmap::masked, yakBitmap::showType eShowTypeForActors = yakBitmap::masked );
  //@member DrawActors | Draws the animap's actors
  virtual void DrawActors( pageHandle_t pagPage, screenCoordX_t scxLeft, screenCoordY_t scyTop,
    tileCoordX_t tcxLeft, tileCoordY_t tcyTop, int iDrawWidth, int iDrawHeight,
    yakBitmap::showType eShowTypeForActors = yakBitmap::masked );
  //@member SmartRefresh | Does a dirty-rectangle refresh of the last displayed map section
  virtual void SmartRefresh( pageHandle_t pagPage, tileCoordX_t tcxLeft, tileCoordY_t tcyTop, bFlag bfShouldReset = true,
    yakBitmap::showType eShowTypeForTiles = yakBitmap::masked, yakBitmap::showType eShowTypeForActors = yakBitmap::masked );

  //@cmember Loads a tile into a given tile slot.
  bFlag bfLoadTile( int iPosition, const char * CszFilename, yakLib * plib = NULL );
  //@cmember Loads only the map grid data of the map from disk (does not load tiles)
  virtual bFlag bfLoadMapDataOnly( const char * CszFilename, yakLib * plibForMap );
  //@cmember Loads the animap from disk
  virtual bFlag bfLoad( const char * CszFilename, yakLib * plibForTiles, yakLib * plibForMap );
  //@cmember Saves the animap to disk
  virtual bFlag bfSave( const char * CszFilename ) const;

  //@cmember Sets a given square to a given tile and terrain type
  void SetSquareToTile( tileCoordX_t tcx, tileCoordY_t tcy, word wTileNumber, terrainType_t terNewTerrain );
  //@cmember Sets the whole map to a given tile and terrain type
  void SetMapToTile( word wTileNumber, terrainType_t terNewTerrain );
  //@cmember Randomizes the tile and terrain type of all map squares
  void Randomize( void );
};

#endif
