#ifndef GADGETS_H

#define GADGETS_H

#include "animbitm.h"
#include "xlib.h"
#include "yakmouse.h"

#define HELP_MASK 4096
#define NO_COMMANDCHAR 0
#define NO_IDENTITY 0
//@doc

//@type gadgetID_t | ID number of a gadget.
typedef int gadgetID_t;

//@enum gadgetRequest | ID of the type of request made from a gadget
enum gadgetRequest {
  noRequest     = 0,    //@emem No request made
  normalRequest = 1,    //@emem Normal usage request made
  helpRequest   = 2     //@emem Request for help on the gadget (largely unimplemented)
};

class gadgetList;
/*
@enum gadget::touchType | How the gadget reacts to a mouse click
@emem normal | Is activated once per mouse click
@emem touchPad | Is activated as long as button is down
*/

/*
@enum gadget::gadgetType | Type of gadget (primitive typechecking)
@emem buttonType | <c button>
@emem toggleSwitchType | <c toggleSwitch>
@emem sliderType | <c slider>
@emem inputLineType | <c inputLine>
*/

/*
@class A gadget is a simple GUI element that interprets user action in a predefined way.
@base public | yakObject
*/
class gadget : public yakObject
{
friend class gadgetList;
public:
  enum touchType {
    normal    = 0,
    touchPad  = 1
  };
  enum gadgetType {
    buttonType        = 1,
    toggleSwitchType  = 2,
    sliderType        = 3,
    inputLineType     = 4
  };
protected:
  //@access Protected Members
  //@cmember X position on screen
  screenCoordX_t scx;
  //@cmember Y position on screen
  screenCoordY_t scy;
  //@cmember Character which activates the gadget
  char cCommandChar;
  //@cmember How the gadget reacts to a mouse click
  touchType eTouchType;
  //@cmember Type of gadget
  gadgetType eGadgetType;
  //@cmember ID number of the gadget (return value for <c button> and its derivatives, help value for others)
  gadgetID_t gidIdentity;  
public:
  //@access Public Members
  //@cmember X position of gadget relative to its <c gadgetList>
  int iRelX;
  //@cmember Y position of gadget relative to its <c gadgetList>
  int iRelY;
  //@member gadget( parms... ) | Specific constructor
  gadget( screenCoordX_t scxNew, screenCoordY_t scyNew, char cNewCommandChar,
    gadgetID_t gidNewIdentity = NO_IDENTITY,
    gadget::touchType eNewTouchType = gadget::normal );
  //@cmember Default constructor
  gadget( void );
  //@cmember Default destructor
  virtual ~gadget( void );
  //@cmember Draws the gadget to screen at its last drawn position
  virtual void Draw( pageHandle_t pagPage = VisiblePageOffs );

  //the following virtual functions are the ones to redefine for your
  //own gadget types
  //@access Functions to redefine in your own gadgets
  //@cmember Draws the gadget to a particular screen location
  virtual void Draw( pageHandle_t pagPage, screenCoordX_t scxNew, screenCoordY_t scyNew ) = 0;
  //@cmember Returns selection status of the gadget
  gadgetRequest grIsSelected( void ) const;
  //@cmember Checks for keyboard-based selection of the gadget
  virtual gadgetRequest grIsKeyboardSelected( void ) const;
  //@cmember Checks for mouse-based selection of the gadget
  virtual gadgetRequest grIsMouseSelected( void ) const = 0;
  //@cmember Executed when the gadget is selected
  virtual void Activate( void ) = 0;
  //@cmember Executed when gadget is done doing its job
  virtual void DeActivate( void );

  //@cmember Dumps gadget data (switch position, etc) to a buffer
  virtual void DumpToBufferCounter( byte ** ppbCounter ) const;
  //@cmember Loads gadget data (switch position, etc) to a buffer
  virtual void LoadFromBufferCounter( byte ** ppbCounter );
  //@cmember Checks the gadget for selection status
  virtual gadgetID_t gidStatus( void );
  //@member eGetTouchType() | Returns eTouchType value
  inline touchType eGetTouchType() const { return eTouchType; };
  //@member eGetGadgetType() | Returns eGadgetType value
  inline gadgetType eGetGadgetType() const { return eGadgetType; };
};

/*
@class A non-animated button that simply accepts a mouse-click or keyboard to send a
normal <t gadgetRequest>.
@base public | gadget
@base public | animBitmap
*/
class button : public gadget, public animBitmap
{
public:
  //@access Public Members
  //@member button( parms... ) | Specific constructor
  button( screenCoordX_t scxNew, screenCoordY_t scyNew, char cNewCommandChar, gadgetID_t gidNewIdentity,
    const char * CszFilename, yakLib * plib = NULL,
    gadget::touchType eTouchType = gadget::normal );
  virtual void Draw( pageHandle_t pagPage, screenCoordX_t scxNew, screenCoordY_t scyNew );
  virtual gadgetRequest grIsMouseSelected( void ) const;
  virtual void Activate( void );
  virtual void DeActivate( void );
};

/*
@class A line of text which acts as a <c button>
@base public | gadget
*/
class label : public gadget
{
#define STANDARD_FGCOLOR 15
#define STANDARD_BGCOLOR 0
#define LABEL_MAXLENGTH 80
public:
  //@access Public Members
  //@cmember Text of the label
  const char * CszText;
  //@cmember Foreground text color
  color_t colFgColor;
  //@cmember Background text color
  color_t colBgColor;
  //@member label( parms... ) | Specific constructor
  label( screenCoordX_t scxNew, screenCoordY_t scyNew, char cNewCommandChar, gadgetID_t gidNewIdentity,
    const char * CszNewText, color_t colNewFgColor = STANDARD_FGCOLOR,
    color_t colNewBgColor = STANDARD_BGCOLOR,
    gadget::touchType eGadgetFlags = gadget::normal );
  virtual void Draw( pageHandle_t pagPage, screenCoordX_t scxNew, screenCoordY_t scyNew );
  virtual gadgetRequest grIsMouseSelected( void ) const;
  virtual void Activate( void );
  virtual void DeActivate( void );
};

/*
@class A multi-position switch which changes position when selected.  The switch position
can then be read at any time.
@base public | button
*/
class toggleSwitch : public button
{
private:
  //@access Private Members
  int iPosition;
public:
  //@access Public Members
  //@member iCurrentPosition | Returns the current position of the toggleSwitch
  inline int iCurrentPosition( void ) const { return iPosition; };
  //@cmember Resets the toggleSwitch to a "zero" position
  void Reset( void );
  //@member toggleSwitch( parms... ) | Specific constructor
  toggleSwitch( screenCoordX_t scxNew, screenCoordY_t scyNew, char cNewCommandChar, gadgetID_t gidNewIdentity,
    const char * CszFilename, yakLib * plib = NULL,
    gadget::touchType iNewGadgetFlags = gadget::normal );
  virtual void Activate( void );
  virtual void DeActivate( void );
  virtual void DumpToBufferCounter( byte ** ppbCounter ) const;
  virtual void LoadFromBufferCounter( byte ** ppbCounter );
};

/*
@class a sliding bar which can be dragged by the mouse and used to get a range input from
the user.

@base public | gadget
*/
class slider : public gadget
{
#define STANDARD_BORDERCOLOR 10
#define STANDARD_INDICATORCOLOR 25
private:
  //@access Private Members
  int iPosition;
public:
  //@access Public Members
  //@cmember Returns current position of the slider
  int iCurrentPosition( void ) const;
  //@cmember Sets new position of the slider bar
  void SetPosition( int iNewPosition );
  //@cmember Width of the slider bar
  int iWidth;
  //@cmember Height of the slider bar
  int iHeight;
  //@cmember Minimum value of the slider bar
  int iMinValue;
  //@cmember Maximum value of the slider bar
  int iMaxValue;
  //@cmember Color of the bar
  color_t colBarColor;
  //@cmember Color of the border
  color_t colBorderColor;
  //@cmember color of the indicator
  color_t colIndicatorColor;
  //@member slider( parms... ) | Specific constructor
  slider( screenCoordX_t scxNew, screenCoordY_t scyNew, char cNewCommandChar, gadgetID_t gidNewIdentity,
    int iNewWidth, int iNewHeight, int iNewMinValue, int iNewMaxValue,
    color_t colBarColor = STANDARD_BGCOLOR,
    color_t colBorderColor = STANDARD_BORDERCOLOR,
    color_t colIndicatorColor = STANDARD_INDICATORCOLOR);
  slider( void );
  virtual void Draw( pageHandle_t pagPage, screenCoordX_t scxNew, screenCoordY_t scyNew );
  virtual gadgetRequest grIsMouseSelected( void ) const;
  virtual gadgetRequest grIsKeyboardSelected( void ) const;
  virtual void Activate( void );
  virtual void DumpToBufferCounter( byte ** ppbCounter ) const;
  virtual void LoadFromBufferCounter( byte ** ppbCounter );
};

//@type inputLine::inputLineString | Array of characters the size of an inputLine's internal data
/*
@enum inputLine::maskType | Type of characters allowed to be input to the inputLine
@emem letters | Allows only alphabetic characters
@emem numbers | Allows only digits
@emem alphanumeric | Allow letters and numbers
@emem symbols | Allows symbols
@emem all | Allows all typable characters
*/

/*
@class A blank line which, when selected, allows keyboard user input.
@base public | gadget
*/
class inputLine : public gadget
{
#define MAX_INPUTLINE_LENGTH 80
public:
  //@access Public Members
  typedef char inputLineString[ MAX_INPUTLINE_LENGTH ];
  enum maskType {
    letters       = 1,
    numbers       = 2,
    alphanumeric  = 3,
    symbols       = 4,
    all           = 7
  };
  //@cmember Type of characters allowed
  maskType eMask;
  //@cmember Background color
  color_t colBgColor;
  //@cmember Foreground color
  color_t colFgColor;
  //@cmember Width of the input line in pixels
  int iWidthInPixels;
  //@cmember Position of the input cursor in the input string
  int iPosition;
  //@cmember Current contents of the input string
  inputLineString acContents;
  //@member inputline( parms... ) | Specific constructor
  inputLine( screenCoordX_t scxNew, screenCoordY_t scyNew, char cNewCommandChar, gadgetID_t gidNewIdentity,
    int iNewWidth, color_t colNewFgColor, color_t colNewBgColor,
    const char * CszInitialContents, maskType eNewMask = all );
  virtual void Draw( pageHandle_t pagPage, screenCoordX_t scxNew, screenCoordY_t scyNew );
  virtual gadgetRequest grIsMouseSelected( void ) const;
  virtual void Activate( void );
  virtual void DumpToBufferCounter( byte ** ppbCounter ) const;
  virtual void LoadFromBufferCounter( byte ** ppbCounter );
};

//Node definitions Follow------------------------------------------>
//@type gadgetNode | An element in a <c gadgetList>
typedef pyListNode gadgetNode;
//list definitions follow----------------------------------------->

/*
@class A list of gadgets, which can all be drawn, checked, and utilized at once.  Members of
a gadgetList maintain their relative positions.  Useful for menus, etc.

@base public | pyList
*/
class gadgetList : public pyList
{
public:
  //@access Public Members
  //@cmember Checks all <c gadget>s in the list for selection and returns the ID of the
  //first selected <c gadget>
  virtual gadgetID_t gidStatus( void );
  //@cmember Draws all <c gadget>s in the list at the given screen coordinate.
  void Draw( pageHandle_t pagPage, screenCoordX_t scxNew, screenCoordY_t scyNew );
  //@cmember Draws all <c gadget>s in the list at the last drawn location
  void Draw( pageHandle_t pagPage = VisiblePageOffs );
  //@cmember Dumps the contents of all the gadgets into an existing buffer
  void DumpContentsToBuffer( byte * pb ) const;
  //@cmember Loads the contents of all the gadgets from an existing buffer
  void LoadContentsFromBuffer( byte * pb );
  //@cmember default destructor
  virtual ~gadgetList( void );
};
#endif
