#ifndef YAKBITM_H

#define YAKBITM_H

#include "stddefs.h"
#include "yakems.h"
#include "yaklib.h"

//@doc

//@struct rolledBltData | A normal rectangular bitmap image
struct rolledBltData {
  short sWidth;           //@field Width in pixels
  short sHeight;          //@field Height in pixels
  byte abImage[];          //@field Bitmap pixels in normal format
};

//@struct unRolledBltData | A bitmap image in xlib-compatible format
struct unRolledBltData {
  byte bNibbleWidth;      //@field Width in pixels divided by 4 and rounded up
  byte bHeight;           //@field Height in pixels
  byte abImage[];         //@field Bitmap pixels in interleaved format
};

class animBitmap;
class yakBitmapMaintenance;

/*
@enum yakBitmap::bitmapType | Ways in which the yakBitmap can be displayed
@emem normal | Normal
@emem compiled | yakBitmap is "compiled" into a machine executable form (for speed).  Not recommended
@emem collidable | yakBitmap can be displayed in solid color and can be tested for collisions
@emem zoomable | yakBitmap can be scaled to different dimensions
*/

/*
@enum yakBitmap::showType | How the yakBitmap will be displayed
@emem solid | Directly copies all bits
@emem masked | Shown as transparent (color 0 is transparent)
@emem clipped | Clipped to the current clipping rectangle
@emem maskedAndClipped | Masked and clipped
*/
/*
@class The basic building block of all yakIcons images, the yakBitmap is a bitmap which can
be easily manipulated with its member functions.

@base public | yakObject
*/
class yakBitmap : public yakObject {
  friend class animBitmap;
  friend class yakBitmapMaintenance;
private:
  //@access Private members
  #ifdef __386__
  byte * emdUnrolledPic;
  byte * emdCollisionMap;
  byte * emdZoomMap;
  #else
  //@cmember The interleaved bytes of the bitmap
  emsData emdUnrolledPic;
  //@cmember The collision bitmap of the yakBitmap
  emsData emdCollisionMap;
  //@cmember Contains the unrolled bitmap (for zooming)
  emsData emdZoomMap;
  #endif
  //@cmember Width of the yakBitmap in pixels
  short sWidth;
  //@cmember Height of the yakBitmap in pixels
  short sHeight;
  //@cmember Creates a collision map (initializes emdCollisionMap)
  void MakeCollisionMap( void );
  //@cmember Creates a zooming map (initializes emdZoomMap)
  void MakeZoomMap( void );
public:
  enum bitmapType {
    normal = 0,
    compiled = 1,
    collidable = 2,
    zoomable = 4
  };
  //@member iGetWidth | Returns width of the yakBitmap in pixels
  inline int iGetWidth( void ) const { return sWidth; };                        //in pixels
  //@member iGetHeight | Returns height of the yakBitmap in pixels
  inline int iGetHeight( void ) const { return sHeight; };               //in pixels
  //@cmember How the yakBitmap can be displayed
  bitmapType eBitmapType;
  //@cmember Default constructor
  yakBitmap();
  //@cmember Default destructor
  virtual ~yakBitmap();
  //@cmember Color of a given pixel in the yakBitmap
  color_t colGetPixel( int iX, int iY ) const;
  //@cmember Sets the color of a given pixel in the yakBitmap
  void SetPixel( int iX, int iY, color_t colColor );
  //@cmember Saves the yakBitmap to disk
  bFlag bfSave( const char * CszFilename ) const;
  //@member GetFromScreen | Gets the yakBitmap data from the screen
  void GetFromScreen( pageHandle_t pagPage, screenCoordX_t scxLeft,
    screenCoordY_t scyTop, screenCoordX_t scxRight,
    screenCoordY_t scyBottom, bitmapType eNewBitmapType = normal );
  //@cmember Uses data in memory for the yakBitmap data
  bFlag bfUseData( byte * pbMyPointer, bitmapType eNewBitmapType = normal );
  //@cmember loads the yakBitmap from disk
  bFlag bfLoad( const char * CszFilename, yakLib * plib = NULL, bitmapType eNewBitmapType = normal );

  enum showType {
    solid = 0,              //color zero as color zero
    masked = 1,             //color zero as transparent
    clipped = 2,            //clipped to the clipping rectangle
    maskedAndClipped = 3,   //color zero transparent, and clipped
  };
  //@cmember Shows the yakBitmap to video memory
  void Show( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, showType eShowType = masked ) const;
  //@cmember Shows the yakBitmap to video memory, scaled to given dimensions
  void ShowZoomed( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, int iNewWidth, int iNewHeight, showType eShowType = masked ) const;
  //@cmember Shows the yakBitmap to video memory in a solid color
  void ShowCollision( pageHandle_t pagPage, screenCoordX_t scx, screenCoordY_t scy, color_t colColor ) const;
  //@cmember Copies a section of video memory the size of the yakBitmap from another page
  void Hide( pageHandle_t pagTo, screenCoordX_t scx, screenCoordY_t scy, pageHandle_t pagFrom ) const;

  //@cmember Checks for collision with another yakBitmap
  bFlag bfHitXY( screenCoordX_t scxThis, screenCoordY_t scyThis, yakBitmap * pybmTarget, screenCoordX_t scxTarget, screenCoordY_t scyTarget ) const;
};

unRolledBltData * punrUnrollBlt( rolledBltData * prol, short * psWidth, short * psHeight,
  int * piNewSize );
rolledBltData * prolRollBlt( unRolledBltData * punr, short sWidthInPixels );


#endif
