#ifndef YAKLIB_H

#define YAKLIB_H

#include "stddefs.h"

#include <fstream.h>

#define HEADERTITLE "yak\0"

//@doc
/*
@class Simple header structure that contains information about a yakIcons file.
*/
class yakHeader
{
public:
  enum fileType {
    yakBitmapFile,
    mapFile,
    paletteFile,
    fontFile,
    libFile
  };
private:
  //@access Private Members
  char acTitle[4];
public:
  //@access Public Members
  //@cmember Major revision number of yakIcons that this file
  byte bMajorRevision;
  //@cmember Minor revision number of yakIcons that generated this file
  byte bMinorRevision;
  //@cmember Type of file (yakHeader::fileType, defined as short to keep size consistent between compilers)
  short eFileType;
  //@cmember Included for future expansion.  Unused at this time
  word wFutureExpansion;
  //@cmember Specific constructor
  yakHeader( fileType eNewFileType );
  //@cmember Checks to make sure this is a valid header
  bFlag bfIsOk( void );
};

/*
@class Contains information about an entry in a yakLib
*/
class directoryEntry
{
public:
  struct yakTime
  {
  //ff_ftime 15.....1110........54.......0
  // ͹
  //                               ͼ
  //                                              Hours     Minutes   Seconds/2
    word wSecondsDiv2 : 5;
    word wMinutes : 6;
    word wHours : 5;
  };

  struct yakDate
  {
  //ff_fdate 15..........98.....54.......0
  // ͹
  //         ͼ
  //        Years since 1980  Month     Day
    word wDay : 5;
    word wMonth : 4;
    word wYearsSince1980 : 7;
  };
  //@cmember Filename of the entry
  filenameString fnamFilename;
  //@cmember Offset of entry in the owning <c yakLib> file
  unsigned long lIndex;
  //@cmember Size of the entry in bytes
  unsigned long lSize;
  //@cmember Date of the entry
  yakDate datDate;
  //@cmember Time of the entry
  yakTime timTime;
  //@cmember Default constructor
  directoryEntry();
};

class yakLibMaintenance;

/*
@class One of the most useful yakIcons classes, the yakLib holds many data files in one
large chunk, similar to a zip file but with no compression.
*/
class yakLib
{
friend class yakLibMaintenance;
friend void bfLoadFile();
friend void bfLoadFileSize();
  //@access Private Members
  //@cmember Disk file containing the yakLib data
  fstream fsLibData;
  //@cmember Filename of the library, including path if necessary
  char acLibFilename[ 255 ];
  //@cmember Number of entries in the library
  short sNumberOfEntries;
  //@cmember Index of the directory in the library
  long lDirectoryIndex;
  //@cmember An array which holds all the entries in the library
  directoryEntry * pdenDirectory;
  //@cmember Sets up lDirectoryIndex
  void getDirectoryIndex( void );
  //@cmember Sets up pdenDirectory
  void getDirectory( void );
public:
  //@access Public Members
  //@cmember Default constructor
  yakLib();
  //@cmember Specific constructor (loads from disk)
  yakLib( const char * CszFilename );
  //@cmember Default destructor
  ~yakLib();
  //@cmember Returns the size of a given file
  long lFileSize( const char * CszFilename );
  //@cmember Opens the yakLib to the given filename
  bFlag bfOpen( const char * CszFilename );
  //@cmember Closes the yakLib.  Done automatically when yakLib is destroyed.
  void Close( void );
  //@cmember Creates an empty yakLib with the given filename
  bFlag bfCreate( const char * CszFilename );
  //@cmember Adds a file to the yakLib
  bFlag bfAdd( const char * CszFilename );
  //@cmember Extracts a file from the yakLib to DOS
  bFlag bfExtract( const char * CszFilename );
  //@cmember Gets a directory entry for the given filename
  bFlag bfGetEntry( const char * CszFilename, directoryEntry * pden );
  //@cmember Deletes a file from the yakLib
  bFlag bfDelete( const char * CszFilename );
  //@cmember Loads a file to memory, getting size information
  bFlag bfLoadSize( const char * CszFilename, byte ** ppbTo, long * plSize );
  //@cmember Loads a file to memory
  bFlag bfLoad( const char * CszFilename, byte ** ppbTo );
  //@cmember Returns the file handle of the yakLib file
  inline fstream * pfstrGetFile( void ) { return &fsLibData; };
};

bFlag bfLoadDos( const char * CszFilename, byte ** ppbTo );
bFlag bfLoadDosSize( const char * CszFilename, byte ** ppbTo, long * plSize );
bFlag bfLoadFile( const char * CszFilename, byte ** ppbTo, yakLib * plib );
bFlag bfLoadFileSize( const char * CszFilename, byte ** ppbTo, long * plSize,
  yakLib * plib );
bFlag bfLoadYakFile( const char * CszFilename, byte ** ppbTo, yakLib * plib,
  yakHeader::fileType eFileType, byte bMinMajorRevision,
  byte bMinMinorRevision );
bFlag bfLoadYakFileSize( const char * CszFilename, byte ** ppbTo, long * plSize,
  yakLib * plib, yakHeader::fileType eFileType, byte bMinMajorRevision,
  byte bMinMinorRevision );
unsigned long ulCRC32( const char * CszFilename, yakLib * plib );  
void AppendFileSuffix( const char * CszOldName, const char * CszSuffix, filenameString fnamNew );

/*
and primitive dos filefind functions, shelled in yakIcons because they
differed between Borland and Watcom.
*/
/*
@struct findStructure | Primitive DOS filefind function data, shelled to a structure because filefind commands
differed between yakIcons and DOS.
*/
struct findStructure {
  char acReserved[21];     //@field Reserved for use by DOS
  byte bAttributes;         //@field Attribute byte for file 
  word wTime;               //@field Time of last write to file
  word wDate;               //@field Date of last write to file
  unsigned long  ulSize;  //@field Length of file in bytes */
#if defined(__OS2__) || defined(__NT__)
//note that yak3 does not work under NT, OS2 well.  This is for expansion
  char acName[256];         /* null-terminated filename */
#else
  char acName[13];          //@field Null-terminated filename */
#endif

  bFlag bfFindFirst( const char * CpcFilename, byte bAttributes );
  bFlag bfFindNext( void );
  void CloseFind( void );
};

//@mfunc bFlag | findStructure | bfFindFirst | Finds the first file which matches the search data
//@mfunc bFlag | findStructure | bfFindNext | Finds the next file which matches the search data
//@mfunc void | findStructure | CloseFind | Ends a searchy
#endif
