#ifndef YAKLIST_H
#define YAKLIST_H

#include <stddef.h>
#include "stddefs.h"
//@doc

/*
@class A node in a linked-list structure.
*/
class pyListNode {
public:
  //@access Public Members
  //@cmember Pointer to the next node in the list
  pyListNode *  pnodNext;
  //@cmember Pointer to the first node in the list
  pyListNode *  pnodPrev;
  //@cmember Pointer to the <c yakObject> referred to by this node
  yakObject * pyData; 
  //@cmember Specific constructor
  pyListNode( yakObject * pyThisData, pyListNode * pnodNextNode );
  //@cmember Default constructor
  pyListNode();
  //@cmember Default destructor
  virtual ~pyListNode( void );
  //@cmember Detaches this node from its list
  void Detach( void );
};

/*
@class A generic linked-list structure which can hold pointers to <c yakObject>s.
*/
class pyList {
public:
  //@access Public Members
  //@cmember Head of the list
  pyListNode nodHead;
  //@cmember Tail of the list
  pyListNode nodTail;
  //@cmember Default constructor
  pyList();
  //@cmember Default destructor
  virtual ~pyList( void );
  //@cmember Returns a pointer to the first object in the list
  yakObject * pyHeadData() const;
  //@cmember Returns a pointer to the last object in the list
  yakObject * pyTailData() const;
  //@cmember Adds an object to the head of the list
  void AddHead( yakObject * pyNewData );
  //@cmember Adds an object to the tail of the list
  void AddTail( yakObject * pyNewData );
  //@cmember Adds a node to the list before the specified node
  void AddNode( pyListNode * pnodNextNode );
  //@cmember empties the list, detaching and destroying all nodes
  void EmptyList();
  //@cmember detaches and destroys the last node in the list
  void RemoveTail();
  //@cmember detaches and destroys the first node in the list
  void RemoveHead();
  //@cmember removes the node containing the given yakObject
  void Remove( yakObject * py );
  //@cmember Checks to see if the list is empty
  bFlag bfIsEmpty( void ) const;
  //@cmember Returns the number of nodes in the list
  int iNumberInList( void ) const;
};

/*
@class An iterator class which points to a <c yakList> and can easily move about in it.
*/
class pyListIterator {
  //@access Private Members
  //@cmember Step number in the list
  int iStepNumber;
public:
  //@access Public Members
  //@cmember Pointer to the owning <c pyList>
  const pyList * plstOwning;
  //@cmember Pointer to the current <c pyListNode> the iterator is looking at
  pyListNode *  pnodCurrent;
  //@cmember Determines whether or not the iterator loops to the other end of the list if
  //incremented or decremented past the end of the list
  bFlag bfLooped;
  //@cmember Initializes the iterator to a given <c pyList>
  void Initialize( const pyList * plstFrom );
  //@cmember Specific constructor (initializes the iterator to the given <c pyList>
  pyListIterator( const pyList * plstFrom );
  //@cmember Default constructor
  pyListIterator();
  //@member iGetStepNumber | Returns the step number of the iterator
  inline int iGetStepNumber() const { return( iBound( iStepNumber, 0, plstOwning->iNumberInList() - 1 ) ); };
  //@member bfHasNodesAhead | Determines whether or not the iterator is at the end of the list
  inline bFlag pyListIterator::bfHasNodesAhead(
    void
  ) const
  {
    //if the current node is the tail, return failure
    if ( pnodCurrent == &( plstOwning->nodTail ) ) {
      return failure;
    }
    else {
      return success;
    }
  };
  //@cmember Determines whether or not the iterator is at the beginning of the list
  bFlag bfHasNodesBehind() const;
  //@member pyCurrent | Returns a pointer to the <c yakObject> in the current <c pyListNode>
  inline yakObject * pyCurrent() const { return pnodCurrent->pyData; };
  //@cmember Increments the iterator
  yakObject * operator ++();
  //@cmember Decrements the iterator
  yakObject * operator --();
  //@cmember Resets the iterator to the head of the <c pyList>
  void SetToHead();
  //@cmember Resets the iterator to the tail of the <c pyList>
  void SetToTail();
};

/*
@class A generic iterator which automatically adjusts its return type to match its
given template type.

@tcarg class | T | Class of the returned pointer when the iterator returns its current position
*/
template <class T> class ptListIterator : public pyListIterator {
public:
  //@access Public Members
  //@member ptCurrent | Returns a typed pointer to the current <c yakObject>
  T * ptCurrent() const {return (T *)( pnodCurrent->pyData );};
  //@member ptListIterator | Default constructor
  ptListIterator() : pyListIterator() {};
  //@member ptListIterator( pyList * plstFrom ) | Specific constructor
  ptListIterator( const pyList * plstFrom ) : pyListIterator( plstFrom ) {};
};

#endif
