#ifndef YAKSOUND_H
#define YAKSOUND_H

#include "stddefs.h"
#include "yaklib.h"
#include "audio.h"
#include "import.h"
//@doc
class yakSong;
class yakSample;
//@type yakSound::sampleRate_t | Digital sample playback rate
//@type yakSound::volume_t | Volume of playback
//@type yakSound::balance_t | Left/right panning position of playback
//@type yakSound::trackNumber_t | Track number of a song
//@type yakSound::channel_t | Digital playback channel

//@class The basic setup class for digital sound in yakIcons.  It is composed of all
//static members, and is a namespace emulation.
class yakSound {
  //@access Private Members
  friend class yakSong;
  friend class yakSample;
  //@cmember The sound card in use
  static SoundCard scardActive;
public:
  //@access Public Members
  typedef unsigned long sampleRate_t;
  typedef unsigned long volume_t;
  typedef int balance_t;
  typedef int trackNumber_t;
  typedef int channel_t;
  //@cmember Initializes yakSound services
  static bFlag bfInitialize( bFlag bfUseTimerService = false, char * szConfigFilename = NULL );
  //@cmember Sets up number of song, dedicated, and generic playback channels
  static void SetupChannels( int iNewSongChannels, int iNewDedicatedChannels = 0, int iNewGenericChannels = 4 );
  //@cmember Returns the first dedicated sound channel
  static channel_t chanFirstDedicatedSound( void );
  //@cmember Returns the first generic sound channel
  static channel_t chanFirstGenericSound( void );
  //@cmember Returns the total number of channels
  static int iNumChannels( void );
  //@cmember Returns the active/inactive status of the yakSound services
  static bFlag bfGetSoundActive( void );
private:
  static volume_t volMaster;
  //@access Private Members
  //@member yakSound | Private constructor so no yakSound objects can be created
  yakSound() {};
  //@cmember Number of song channels
  static int iSongChannels;
  //@cmember Number of dedicated sample channels
  static int iDedicatedChannels;
  //@cmember Number of generic sample playback channels
  static int iGenericChannels;
  //@cmember Flag whether DSMI is using EMS to store sound data
  static bFlag bfDSMIInEMS;
  //@cmember Flag whether yakSound services are active
  static bFlag bfSoundActive;
};

/*
@class A digital song which contains both digital instrument data and information controlling
that data's playback (ie note pitches, durations, etc).
*/
class yakSong {
  //@access Private Members
  //@cmember Actual song data
  DSM * pmodSongData;
  //@cmember Master playback volume
  static yakSound::volume_t volMaster;
  //@cmember Frees memory in use by the yakSong
  void FreeOwnedMemory( void );
public:
  //@access Public Members
  //@cmember Default constructor
  yakSong( void );
  //@cmember Default destructor
  ~yakSong( void );
  //@cmember Loads a song from disk
  bFlag bfLoad( const char * CszFilename, yakLib * plib = NULL );
  //@cmember Plays a song
  void Play( bFlag bfLooped = true );
  //@cmember Plays a section of the song
  void PlaySection( long lStartPattern, long lEndPattern, bFlag bfLooped = true );
  //@member bFlag bfIsValid( void ) | Tells whether or not a song is loaded and ready to play.
  inline bFlag bfIsValid( void ) { return (bFlag)( pmodSongData != NULL ); };
  //@cmember gets master song volume
  static yakSound::volume_t volGetMasterVolume( void );
  //@cmember Pauses the song
  static void Pause( void );
  //@cmember Resumes a paused song
  static void Resume( void );
  //@cmember Stops all playback
  static void Stop( void );
  //@cmember Fades the song to silence.  Foreground task (program execution pauses while
  //the song fades out)
  static void FadeOut( int iNumberOfSteps, int iStepDelay );
#define TRACK_DEFAULT -1
  //@cmember Sets volume on a track or all tracks
  static void SetVolume( yakSound::volume_t volNew );
};

//@type yakSample::frequency_t | Frequency of sample playback
//@type yakSample::panPosition_t | Left/right panning position of sample playback
/*
@class A simple digital sample class that allows multichannel playback at different stereo
panning positions, volumes, etc.
*/
class yakSample {
friend class yakSound;
public:
  typedef unsigned long frequency_t;
  typedef int panPosition_t;
private:
  //@access Private Members
  //@cmember Digitized sample data
  Sample * pSample;
  //@cmember Default frequency of playback
  frequency_t freqDefault;
  //@cmember Dedicated channel to play back sound (starts at zero and counts up)
  int iDedicatedChannelOffset;
  //@cmember Master sample playback volume
  static yakSound::volume_t volMaster;
  //@cmember Current free playback channel
  static yakSound::channel_t chanCurrentFree;
  //@cmember Frees memory used by the sample
  void FreeOwnedMemory( void );
public:
  //@access Public Members
  //@cmember Default Constructor
  yakSample( void );
  //@cmember Default Destructor
  ~yakSample( void );
#define FREQ_DEFAULT 11025
#define PLAY_DEFAULT -1
  //@cmember Returns the channel the sound is about to use
  yakSound::channel_t chanToUse( void );
  //@cmember Loads sample from disk
  bFlag bfLoad( const char * CszFilename, yakLib * plib = NULL, int iNewDedicatedChannelOffset = PLAY_DEFAULT );
  //@cmember Sets playback frequency of the sample
  void SetFrequency( frequency_t freqNew );
  //@cmember Sets the dedicated channel offset of the sample
  void SetDedicatedChannelOffset( int iNewDedicatedChannelOffset );
  //@cmember Plays the sample through the current sound device
  void Play( yakSound::volume_t volNew = PLAY_DEFAULT, panPosition_t panNew = PAN_MIDDLE, frequency_t freqNew = PLAY_DEFAULT );
  //@cmember Returns the master sample playback volume
  static yakSound::volume_t volGetMasterVolume( void );
  //@cmember Sets the master sample playback volume
  static void SetMasterVolume( yakSound::volume_t volNewMaster );
};

#endif
